//import { csv, scaleLinear, scaleTime, max, timeFormat, extent } from 'd3';

var svg = d3.select('svg');
  
var margin = 70;
var width = 940 - 2 * margin;
var height = 480 - 2 * margin;

d3.csv("data.csv").then(function(data) {

// Creation du graphe + translation mathématique
  var chart = svg.append('g')
    .attr('transform', `translate(${margin}, ${margin})`);

// Axe des y
  //we create the scale
  var yScale = d3.scaleLinear()
    .range([height, 0])
    .domain([0, 40000]);
  	//.domain(extent(value));
  
  //on dessine l'axe
  chart.append('g')
    .call(d3.axisLeft(yScale));

// Y axis
  var xScale = d3.scaleBand()
    .range([0, width])
    .domain(data.map((data) => data.FunnelPoint))
  	//espacement entre les barres
    .padding(0.2);

  chart.append('g')
      .attr('transform', `translate(0, ${height})`)
      .call(d3.axisBottom(xScale));

// Draw gridlines
	chart.append('g')
      .attr('class', 'grid')
      .call(d3.axisLeft()
          .scale(yScale)
          .tickSize(-width, 0, 0)
          .tickFormat(''));
 
// Titles
// y axis title
  	svg.append('text')
      .attr('x', -(height / 2) - margin)
      .attr('y', margin / 3.4)
      .attr('transform', 'rotate(-90)')
      .attr('text-anchor', 'middle')
      .text('Student Count');
// top title
  	svg.append('text')
      .attr('class','title')
      .attr('x', width / 2 + margin)
      .attr('y', 40)
      .attr('text-anchor', 'middle')
      .text('Funnel Volumes');
// x axis title
  	svg.append('text')
      .attr('x', width / 2 + margin)
      .attr('y', height + 110)
      .attr('text-anchor', 'middle')
      .text('Funnel Points');
// Bottom corner text
  	svg.append('text')
      .attr('x', width - margin)
      .attr('y', height + 110)
      .attr('text-anchor', 'start')
      .text('Source: Stack Overflow, 2019');
      
// Bind the elements of the array DOM element
  var barGroups = chart.selectAll()
  .data(data)
  .enter()
  .append('g');

  console.log(data);

// Draw bars linked to each element of the data
  barGroups
    .append('rect')
    .attr('class', 'bar')
    .attr('x', (data) => xScale(data.FunnelPoint))
    .attr('y', (data) => yScale(data.value))
    .attr('height', (data) => height - yScale(data.value))
    .attr('width', xScale.bandwidth());
  
  //Show value on each bar
  barGroups 
      .append('text')
      .attr('class', 'value')
      .attr('x', (data) => xScale(data.FunnelPoint) + xScale.bandwidth() / 2)
      .attr('y', (data) => yScale(data.value) + 30)
      .attr('text-anchor', 'middle')
      .text((data) => `${data.value}`)

// Event Listener: mouse enter
    barGroups.on('mouseenter', function (actual, i) {
      d3.selectAll('.value')
            .attr('opacity', 0)
       
      d3.select(this)
        .transition()
        .duration(300)
        .attr('opacity', 0.6);

      var y = yScale(actual.value);

      line = chart.append('line')
        .attr('id', 'limit')
        .attr('x1', 0)
        .attr('y1', y)
        .attr('x2', width)
        .attr('y2', y);

      barGroups.append('text')
        .attr('class', 'divergence')
        .attr('x', (data) => xScale(data.FunnelPoint) + xScale.bandwidth() / 2)
        .attr('y', (data) => yScale(data.value) + 30)
        .attr('fill', 'white')
        .attr('text-anchor', 'middle')
        .text((data, idx) => {
          var divergence = (data.value - actual.value).toFixed(1)

          let text = ''
          if (divergence > 0) text += '+'
          text += `${divergence}%`

          return idx !== i ? text : ''; 
        })
    })

//Event Listener: mouse leave
    barGroups.on('mouseleave', function () {
      d3.selectAll('.value')
        .attr('opacity', 1)

      d3.select(this)
        .transition()
        .duration(300)
        .attr('opacity', 1)
        .attr('x', (data) => xScale(data.FunnelPoint))
        .attr('width', xScale.bandwidth())

      chart.selectAll('#limit').remove()
      chart.selectAll('.divergence').remove()
  });
})